﻿//  Copyright © Decebal Mihailescu 2013
//  All rights reserved. This source code and binaries are distributed under 
//  Microsoft Public License (Ms-PL)
//  The latest code is available at http://VSDesignPatterns.codeplex.com

//  Redistributions of source code must retain the above copyright notice, 
//  this list of conditions and the following disclaimer. 
//  Redistributions in binary form must reproduce the above copyright notice, 
//  this list of conditions and the following disclaimer in the documentation 
//  and/or other materials provided with the distribution. 

//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
//  PURPOSE. IT CAN BE DISTRIBUTED FREE OF CHARGE AS LONG AS THIS HEADER 
//  REMAINS UNCHANGED.

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration.Install;
using System.Linq;


namespace DesignPatterns
{
    using System.IO;
    using System.Diagnostics;
    using System.Xml;
    using Microsoft.Win32;
    using DesignPatternsWizard;
    using System.Windows.Forms;
    using System.Runtime.InteropServices;
    using System.Xml.Linq;
    using System.Xml.XPath;
    /*
    http://www.mztools.com/articles/2005/MZ2005002.aspx
    delete addin http://www.mztools.com/articles/2006/MZ2006018.aspx
    reset add-in devenv.exe /resetaddin DesignPatterns.Connect http://www.mztools.com/articles/2006/MZ2006014.aspx
    */
    //#pragma warning disable 1591

    [RunInstaller(true)]
    public partial class AddInInstaller : Installer
    {
        private const string ExtNameSpace =
          "http://schemas.microsoft.com/AutomationExtensibility";
        readonly string _addInTargetFolder;
        readonly string _vsPath;
        internal static string VSPath
        {
            get
            {
                try
                {//use the registry to get the path from 
                    //HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\Microsoft\VisualStudio\10.0\Setup\VS@EnvironmentPath
                    string pgf = Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles);
                    pgf = Path.Combine(pgf,
                        string.Format(@"Microsoft Visual Studio {0}\Common7\IDE\DEVENV.exe", ProjectUtils.VSVersion));
                    //RegistryKey.OpenBaseKey(RegistryHive.LocalMachine, RegistryView.Registry32);
                    string regpath = string.Format(@"SOFTWARE\Microsoft\VisualStudio\{0}\Setup\VS", ProjectUtils.VSVersion);
                    using (var reg = Registry.LocalMachine.OpenSubKey(regpath, false))
                    {
                        if (reg != null)
                        {
                            pgf = reg.GetValue("EnvironmentPath", pgf, RegistryValueOptions.None) as string;
                            Trace.TraceWarning("EnvironmentPath doe not exist for VS version {0}", ProjectUtils.VSVersion);
                        }
                        return pgf;
                    }
                }
                catch (Exception ex)
                {
                    Trace.TraceError("Error while getting VS path :{0}", ex.Message);
                    string vsPath = null;
#if VS2008
                    if (File.Exists(@"c:\Program Files (x86)\Microsoft Visual Studio "+ ProjectUtils.VSVersion + @"\Common7\IDE\devenv.exe"))
                        vsPath = @"c:\Program Files (x86)\Microsoft Visual Studio " + ProjectUtils.VSVersion + @"\Common7\IDE\devenv.exe";
                    else//32 Bit windows
                        vsPath = @"c:\Program Files\Microsoft Visual Studio " + ProjectUtils.VSVersion + @"\Common7\IDE\devenv.exe";

#else
                    
                    vsPath = Environment.Is64BitOperatingSystem ? @"c:\Program Files (x86)\Microsoft Visual Studio "+ ProjectUtils.VSVersion + @"\Common7\IDE\devenv.exe" : @"c:\Program Files\Microsoft Visual Studio "+ ProjectUtils.VSVersion + @"\Common7\IDE\devenv.exe";
#endif
                    /*
#if VS2010
                    vsPath = Environment.Is64BitOperatingSystem ? @"c:\Program Files (x86)\Microsoft Visual Studio 10.0\Common7\IDE\devenv.exe" : @"c:\Program Files\Microsoft Visual Studio 10.0\Common7\IDE\devenv.exe";
                    

#endif
#if VS2012
                        vsPath = Environment.Is64BitOperatingSystem ? @"c:\Program Files (x86)\Microsoft Visual Studio 11.0\Common7\IDE\devenv.exe" : @"c:\Program Files\Microsoft Visual Studio 11.0\Common7\IDE\devenv.exe";


#endif
#if VS2013
                    vsPath = Environment.Is64BitOperatingSystem ? @"c:\Program Files (x86)\Microsoft Visual Studio 12.0\Common7\IDE\devenv.exe" : @"c:\Program Files\Microsoft Visual Studio 12.0\Common7\IDE\devenv.exe";
                    
#endif
                     */ 
                    return vsPath;
                }
            }
        }
        public AddInInstaller()
        {

            InitializeComponent();
            _vsPath = VSPath;
            _addInTargetFolder = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments),
                string.Format(@"Visual Studio {0}\Addins", ProjectUtils.VSYear));

            string logPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments), "DesignPatternsInstall_" + ProjectUtils.VSVersion + ".log");

            Trace.Listeners.Add(new TextWriterTraceListener(logPath));
            Trace.TraceInformation("DesignPatterns Install initialized at {0}, Win ver = {1}, Crt dir: '{2}', CLR version ={3}, Is64BitProcess = {4}",
                DateTime.Now, Environment.OSVersion.Version, Environment.CurrentDirectory, RuntimeEnvironment.GetSystemVersion(), Marshal.SizeOf(IntPtr.Zero) == 8);
        }

        /// <summary>
        /// Overrides Installer.Install, which will be executed during install process.
        /// </summary>
        /// <param name="savedState">The saved state.</param>
        public override void Install(IDictionary savedState)
        {

            // Uncomment the following line, recompile the setup
            // project and run the setup executable if you want
            // to debug into this custom action.
#if DEBUG
            Debugger.Break();
#endif
            base.Install(savedState);

            if (!File.Exists(_vsPath))
            {
                if (ProjectUtils.ShowModalDlg("Can't find Visual Studio " + ProjectUtils.VSYear + " IDE at\r\n" + _vsPath + "\r\nDo you want to continue anyway?"
                      , "Warning", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
                {
                    throw new InstallException("User cancelled");
                }
            }
            else
            {
                if (Process.GetProcessesByName("devenv").Length > 0)
                    ProjectUtils.ShowModalDlg("It is recommended that you close\r\nVisual Studio before continuing!", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }


            string assemblyFolder = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location);


            try
            {
                InstallTemplateLibrary(savedState, assemblyFolder);
                RenameDesignPatternsAddIns();
                VSResetAddIn(true);
                SetAddInControlFile(savedState, assemblyFolder);
                VSResetAddIn(false);

            }
            catch (InstallException ex)
            {
                Trace.TraceError(ex.Message);
                ProjectUtils.ShowModalDlg(ex.Message, "InstallException thrown while installing", MessageBoxButtons.OK, MessageBoxIcon.Error);
                throw;
            }
            catch (Exception ex)
            {
                Trace.TraceError(ex.Message);
                ProjectUtils.ShowModalDlg(ex.Message, "Exception thrown while installing", MessageBoxButtons.OK, MessageBoxIcon.Error);
                throw new InstallException(ex.Message, ex);
            }
            finally
            {
                Trace.Flush();
            }
        }

        private void SetAddInControlFile(IDictionary savedState, string assemblyFolder)
        {
            // Parameters required to pass in from installer's custom actions, more see http://msdn.microsoft.com/en-us/library/aa371857.aspx
            string msiDirPath = this.Context.Parameters["MsiSrcDir"];
            string productName = this.Context.Parameters["ProductName"];
            string assemblyName = this.Context.Parameters["AssemblyName"];

            // Setup .addin path and assembly path
            string addinControlFileName = assemblyName + ".Addin";

            Trace.TraceInformation("Install: productName = {0}, VSersion = {1}, addinControlFileName = {2}, assemblyFolder = {3}",
                productName, ProjectUtils.VSVersion, addinControlFileName, assemblyFolder);
            string addinAssemblyFileName = assemblyName + ".dll";
            //change and deploy the .Addin file
            string sourceFile = Path.Combine(assemblyFolder, addinControlFileName);
            // remove readonly file attr
            FileAttributes fa = File.GetAttributes(sourceFile);
            if ((uint)(fa & FileAttributes.ReadOnly) != 0)
            {
                fa ^= FileAttributes.ReadOnly;// remove read only
                File.SetAttributes(sourceFile, fa);
                // ProjectUtils.ShowModal("file " + sourceFile, "RemoveReadOnlyAttribute", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            Trace.TraceInformation("Install: loading {0}", sourceFile);

            XDocument doc = XDocument.Load(sourceFile);
            XmlReader reader = doc.CreateReader();
            XmlNamespaceManager xnm = new XmlNamespaceManager(reader.NameTable);
            xnm.AddNamespace("def", ExtNameSpace);

            // Update Addin/Assembly node
            XElement node = doc.Root.XPathSelectElement("/def:Extensibility/def:Addin/def:Assembly", xnm);
            if (node != null)
            {
                node.Value = Path.Combine(assemblyFolder, addinAssemblyFileName);
                Trace.TraceInformation("Install: changed assembly node");
            }
            // Update VS version node
            node = doc.Root.XPathSelectElement("/def:Extensibility/def:HostApplication/def:Version", xnm);
            if (node != null)
            {
                node.Value = ProjectUtils.VSVersion;
                Trace.TraceInformation("Install: setted the correct VS version");
            }
            // Update ToolsOptionsPage/Assembly node
            node = doc.Root.XPathSelectElement("/def:Extensibility/def:ToolsOptionsPage/def:Category/def:SubCategory/def:Assembly", xnm);
            if (node != null)
            {
                node.Value = Path.Combine(assemblyFolder, addinAssemblyFileName);
                Trace.TraceInformation("Install: changed subcategory node");
            }

            reader.Close();
            doc.Save(sourceFile);
            Trace.TraceInformation("Install: saved {0}", sourceFile);

            string targetFile = Path.Combine(_addInTargetFolder, assemblyName + ".Addin");
            File.Copy(sourceFile, targetFile, true);
            Trace.TraceInformation("Install: copied '{0}' to '{1}'", sourceFile, targetFile);
            // Save AddinPath to be used in Uninstall or Rollback
            savedState.Add("AddinPath", targetFile);
        }

        private void RenameDesignPatternsAddIns()
        {
            DirectoryInfo dirInfo = new DirectoryInfo(_addInTargetFolder);
            if (!dirInfo.Exists)
            {
                Trace.TraceInformation("Install: creting dir {0}", _addInTargetFolder);
                dirInfo.Create();
            }
            //rename other addins to remove conflicts
            FileInfo[] files = dirInfo.GetFiles();
            foreach (FileInfo fi in files)
            {
                try
                {
                    if (fi.Extension.EndsWith("addin", StringComparison.CurrentCultureIgnoreCase) && fi.Name.ToLower().Contains("designpatterns"))
                    {
                        string bakFile = fi.FullName + "_";
                        if (File.Exists(bakFile))
                            File.Delete(bakFile);
                        fi.MoveTo(bakFile);
                    }
                }
                catch (Exception ex)
                {

                    Trace.TraceWarning("Install: Exception while moving .addin files {0}", ex.Message);
                }

            }
        }

        private void InstallTemplateLibrary(IDictionary savedState, string assemblyFolder)
        {
            DirectoryInfo dirInfo = new DirectoryInfo(assemblyFolder);
            string baseDir = dirInfo.Parent.FullName;
            string libPath = Path.Combine(baseDir, "TemplateLibrary");


            try
            {
                // enable it when "TemplateLibrary.zip" is not part of the msi
                // Parameters required to pass in from installer
                string msiDirPath = this.Context.Parameters["MsiSrcDir"];
                Trace.TraceInformation("msiDirPath = {0}", msiDirPath);
                if (msiDirPath.EndsWith("\\\\"))
                    msiDirPath = msiDirPath.Remove(msiDirPath.Length - 1);//removes last '\'
                DirectoryInfo dinf = Directory.GetParent(msiDirPath);//.Parent;
                FileInfo fi = dinf.GetFiles("TemplateLibrary.zip", SearchOption.TopDirectoryOnly).FirstOrDefault();
                if (fi == null)
                    fi = dinf.Parent.GetFiles("TemplateLibrary.zip", SearchOption.TopDirectoryOnly).FirstOrDefault();

                if (fi == null)
                {
                    ProjectUtils.ShowModalDlg("No TemplateLibrary.zip in the folder\r\n" + dinf.FullName, "Can't find TemplateLibrary.zip", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    Trace.TraceError("No TemplateLibrary.zip in the folder\r\n" + dinf.FullName);
                    throw new Exception("No TemplateLibrary.zip in " + dinf.FullName);
                }
                else
                {

                    string libdest = Path.Combine(baseDir, "TemplateLibrary.zip");
                    Trace.TraceInformation("Copying '{0}' in '{1}'", fi.FullName, libdest);
                    File.Copy(fi.FullName, libdest, true);
                    savedState.Add("TemplateLibZip", libdest);
                    //  ProjectUtils.ShowModal("to " + libdest, "copied TemplateLibrary.zip", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }

                Trace.TraceInformation("Install: unzipping {0} in {1}", libPath, baseDir);
                BaseLanguageProvider.UnzipLibrary(baseDir, libPath);
                RegistryKey reg = Registry.CurrentUser.CreateSubKey(BaseLanguageProvider.AppHive, RegistryKeyPermissionCheck.ReadWriteSubTree);
                if (reg != null)
                {
                    reg.SetValue("TemplateLibPath", libPath, RegistryValueKind.String);
                    reg.Flush();
                    reg.Close();
                    Trace.TraceInformation("set registry TemplateLibPath to {0} ", libPath);
                }
                else
                    Trace.TraceError("Install: can't create registry key {0}", BaseLanguageProvider.AppHive);
                savedState.Add("TemplateLibPath", libPath);

            }
            catch (System.Exception ex)
            {
                Trace.TraceError("Install: exception thrown wile unziping and TemplateLibPath registry set: {0}", ex.Message);
                throw new InstallException("Can't set TemplateLibPath", ex);
            }
        }



        /// <summary>
        /// Overrides Installer.Rollback, which will be executed during rollback process.
        /// </summary>
        /// <param name="savedState">The saved state.</param>
        public override void Rollback(IDictionary savedState)
        {
#if DEBUG
            Debugger.Break();
#endif
            base.Rollback(savedState);

            try
            {
                string fileName = (string)savedState["AddinPath"];
                if (File.Exists(fileName))
                {
                    File.Delete(fileName);
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.ToString());
            }
        }
        protected override void OnCommitted(IDictionary savedState)
        {
            base.OnCommitted(savedState);
            Trace.TraceInformation("called base.OnCommitted");
        }

        private bool VSResetAddIn(bool wait)
        {
            try
            {

                if (File.Exists(_vsPath))
                {//see http://visualstudiohacks.com/tips/when-resetaddin-doesnt-quite-cut-it/ 
                    //or http://social.msdn.microsoft.com/Forums/vstudio/en-US/5f2948f8-6e76-42bf-8685-aa9bfa4f60ef/vs2005-problems-with-devenvexe-resetaddin-addin-command-fileexit-
                    //start vs with no add-in and /resetaddin param, then close VS and start again
                    //use startup info & minimize window
                    ProcessStartInfo psi = new ProcessStartInfo(_vsPath, "/resetaddin " + typeof(DesignPatterns.Connect).FullName + " /Command File.Exit");
                    Process p = Process.Start(psi);
                    psi.WindowStyle = ProcessWindowStyle.Hidden;//change to minimized or hidden
                    if (wait)
                        p.WaitForExit(30000); //wait up to 30 seconds
                    Trace.TraceInformation("VSResetAddIn: ran '{0}'", _vsPath + " /resetaddin " + typeof(DesignPatterns.Connect).FullName + " /Command File.Exit");
                    return true;
                }
                else
                {
                    Trace.TraceWarning("OnCommitted: won't run inexistent '{0}'", _vsPath);
                    return false;
                }
            }
            catch (Exception ex1)
            {

                Trace.TraceWarning("Could not start '{0}' because {1}", _vsPath, ex1.Message);
                return false;
            }
        }
        /// <summary>
        /// Overrides Installer.Uninstall, which will be executed during uninstall process.
        /// </summary>
        /// <param name="savedState">The saved state.</param>
        public override void Uninstall(IDictionary savedState)
        {

            base.Uninstall(savedState);
#if DEBUG
            Debugger.Break();
#endif
            try
            {
                if (Process.GetProcessesByName("devenv").Length > 0)
                    ProjectUtils.ShowModalDlg("It is recommended that you close\r\nVisual Studio before continuing!", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                try
                {
                    string fileName = savedState["AddinPath"] as string;

                    if (File.Exists(fileName))
                    {
                        Trace.TraceInformation("Uninstall: deleting '{0}'", fileName);
                        File.Delete(fileName);
                    }
                    fileName = savedState["TemplateLibZip"] as string;
                    if (fileName != null && File.Exists(fileName))
                    {
                        Trace.TraceInformation("Uninstall: deleting '{0}'", fileName);
                        File.Delete(fileName);
                    }
                    string libPath = savedState["TemplateLibPath"] as string;
                    if (libPath != null && Directory.Exists(libPath))
                        new DirectoryInfo(libPath).Delete(true);

                }
                catch (Exception ex)
                {
                    Trace.TraceWarning("Uninstall: exception thrown '{0}'", ex.Message);

                }
                Registry.CurrentUser.DeleteSubKeyTree(BaseLanguageProvider.AppHive);
                DirectoryInfo dirInfo = new DirectoryInfo(_addInTargetFolder);
                //restore other addins 
                FileInfo[] files = dirInfo.GetFiles();
                foreach (FileInfo fi in files)
                {
                    try
                    {
                        if (fi.Extension.EndsWith("addin_", StringComparison.CurrentCultureIgnoreCase) && fi.Name.ToLower().Contains("designpatterns"))
                        {
                            Trace.TraceInformation("Uninstall: restoring '{0}'", fi.FullName);
                            fi.MoveTo(fi.FullName.Substring(0, fi.FullName.Length - 1));
                        }
                    }
                    catch (Exception ex)
                    {
                        Trace.TraceWarning("Uninstall: exception thrown '{0}' for {1}", ex.Message, fi.FullName);

                    }
                }
                // ProjectUtils.ShowModal(_vsPath, "Uninstall: about to start", MessageBoxButtons.OK, MessageBoxIcon.Information);
                //use startup info & minimize window
                //Process p =Process.Start(_vsPath, "/resetaddin * /Command File.Exit");
                //Trace.TraceInformation("UnInstall: ran '{0}'", _vsPath + " /resetaddin * /Command File.Exit");
                //p.WaitForInputIdle(5000);
                VSResetAddIn(false);

            }
            catch (Exception ex)
            {
                //Debug.WriteLine(ex.ToString());
                Trace.TraceWarning("Uninstall: exception thrown '{0}'", ex.Message);
            }
        }
    }
}
