﻿//  Copyright © Decebal Mihailescu 2013
//  All rights reserved. This source code and binaries are distributed under 
//  Microsoft Public License (Ms-PL)
//  The latest code is available at http://VSDesignPatterns.codeplex.com

//  Redistributions of source code must retain the above copyright notice, 
//  this list of conditions and the following disclaimer. 
//  Redistributions in binary form must reproduce the above copyright notice, 
//  this list of conditions and the following disclaimer in the documentation 
//  and/or other materials provided with the distribution. 

//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
//  PURPOSE. IT CAN BE DISTRIBUTED FREE OF CHARGE AS LONG AS THIS HEADER 
//  REMAINS UNCHANGED.
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Extensibility;
using EnvDTE;
using EnvDTE80;
using Microsoft.VisualStudio.CommandBars;
using System.Resources;
using System.Reflection;
using System.Globalization;

namespace DesignPattterns
{
    using System.Windows.Forms;
    using System.IO;
    using System.Diagnostics;
    using System.Runtime.InteropServices;
    using Microsoft.VisualStudio;
    using Microsoft.VisualStudio.CommandBars;
    using System.Collections.Generic;
    using DesignPatternsWizard;
    using System.ComponentModel;
    using System.Collections;


    static public class DTEUtils
    {    //see http://blogs.msdn.com/b/dr._ex/archive/2007/04/17/using-ivsproffercommands-to-retrieve-a-visual-studio-commandbar.aspx
        [ComImport, Guid("6D5140C1-7436-11CE-8034-00AA006009FA"), InterfaceTypeAttribute(ComInterfaceType.InterfaceIsIUnknown)]
        internal interface IOleServiceProvider
        {
            [PreserveSig]
            int QueryService([In]ref Guid guidService, [In]ref Guid riid,
               [MarshalAs(UnmanagedType.Interface)] out System.Object obj);
        }

        internal static object FindCommandBar(this EnvDTE80.DTE2 applicationObject, Guid guidCmdGroup, uint menuID)
        {
            // Make sure the CommandBars collection is properly initialized, before we attempt to
            // use the IVsProfferCommands service.
            CommandBar menuBarCommandBar = ((CommandBars)applicationObject.CommandBars)["MenuBar"];

            // Retrieve IVsProfferComands via DTE's IOleServiceProvider interface
            IOleServiceProvider sp = (IOleServiceProvider)applicationObject;
            Guid guidSvc = typeof(IVsProfferCommands).GUID;
            Object objService;
            sp.QueryService(ref guidSvc, ref guidSvc, out objService);
            IVsProfferCommands vsProfferCmds = (IVsProfferCommands)objService;
            return vsProfferCmds.FindCommandBar(IntPtr.Zero, ref guidCmdGroup, menuID);
        }

        //see http://zakimirza.wordpress.com/2008/10/27/visual-studio-add-in-creating-context-menus/ for more nodes
        // or http://zakimirza.wordpress.com/visual-studio-2005-extensiblity-command-bars-and-their-controls/
        public const string ITEM = "Item";
        public const string WEB_ITEM = "Web Item";
        public const string MULTI_ITEM = "Cross Project Multi Item";
        public const string FOLDER = "Folder";
        public const string WEB_FOLDER = "Web Folder";
        public const string PROJECT = "Project";
        public const string PROJECT_NODE = "Project Node";
        public const string STANDARD = "Standard";
        public const string MENU_BAR = "MenuBar";

        //see http://stackoverflow.com/questions/3017063/visual-studio-2010-plug-in-adding-a-context-menu-to-the-solution-explorer/3436742#3436742
        public static CommandBar GetCommandBarByItemType(this CommandBars cmdbars, string itemType)
        {
            return cmdbars[itemType];
        }

        public static CommandBar GetCommandBarByItemType(this EnvDTE80.DTE2 applicationObject, string itemType)
        {
            return ((CommandBars)(applicationObject.CommandBars))[itemType];
        }

        public static CommandBarControl AddCommandBar(this Command command, string itemType, int pos, string toolTip)
        {
            CommandBar vsBarFolder = ((CommandBars)command.DTE.CommandBars)[itemType];
            if (vsBarFolder != null)
            {
                Trace.TraceInformation("Making new command and standard tool bar");
                CommandBarControl ctrl = (CommandBarControl)command.AddControl(vsBarFolder, pos);
                ctrl.TooltipText = toolTip;
                return ctrl;
            }
            else
                return null;
        }
        //from http://www.c-sharpcorner.com/UploadFile/mgold/AddIns11292005015631AM/AddIns.aspx
        public static void CreateCopyrightInEveryFileInSolution(this DTE2 applicationObject)
        {
            try
            {
                // Create a copyright text for each code item in the project 
                // Get the array of projects from the active solution
                System.Array theProjects = (System.Array)applicationObject.ActiveSolutionProjects;
                EnvDTE.Project theProject = null;
                // make sure there is an active solution containing projects
                if (theProjects.Length > 0)
                {
                    // This add-in only works for the first project in the solution,
                    // This can be extended for multiple projects by putting a for loop here like
                    // foreach (Project theProject in theProjects)
                    theProject = (EnvDTE.Project)(theProjects.GetValue(0));
                    // Go through each project item in the project
                    foreach (ProjectItem p in theProject.ProjectItems)
                    {
                        Trace.WriteLine("ProjectItem = " + p.Name);
                        // Check to see if its a c# file
                        if (p.Name.Substring(p.Name.Length - 3, 3) == ".cs")
                        {
                            // Open the file as a source code file
                            EnvDTE.Window theWindow = p.Open(Constants.vsViewKindCode);
                            //Get a handle to the new document in the open window
                            TextDocument objTextDoc = (TextDocument)theWindow.Document.Object("TextDocument");
                            EditPoint objEditPoint = (EditPoint)objTextDoc.StartPoint.CreateEditPoint();
                            //Create an EditPoint and add some text.
                            objEditPoint.Insert("// This Code was created by Microgold Software Inc. for educational purposes" + '\n' + "// Copyright Microgold Software Inc. " + DateTime.Now.ToLongDateString() + "\n\n");
                            // Get the filename from the project item and save the file with the copyright information
                            string fName = p.get_FileNames(0);
                            p.SaveAs(fName);
                        }
                        // end if ".cs" file
                    }
                    // end for each project item
                }
                // end if any projects exists in projects collection
            }
            // end try block
            catch (Exception e)
            {
                Trace.WriteLine(e.Message.ToString());
            }
        }


        public static void CreateNewFileWithCopyrightInfo(this DTE2 applicationObject)
        {
            //Create a new text document.
            Window wnd = applicationObject.ItemOperations.NewFile("General\\Text File", "", Constants.vsViewKindCode);

            //Get a handle to the new document.
            TextDocument objTextDoc = (TextDocument)applicationObject.ActiveDocument.Object("TextDocument");

            EditPoint objEditPoint = (EditPoint)objTextDoc.StartPoint.CreateEditPoint();
            //Create an EditPoint and add some text.
            objEditPoint.Insert("// This Code was created by Microgold Software Inc. for educational purposes" + '\n' + "// Copyright Microgold Software Inc. " + DateTime.Now.ToLongDateString());

        }

        //from http://stackoverflow.com/questions/3017063/visual-studio-2010-plug-in-adding-a-context-menu-to-the-solution-explorer
        public static Project GetSelectedProject(this DTE2 applicationObject)
        {
            if (applicationObject.Solution == null || applicationObject.Solution.Projects == null || applicationObject.Solution.Projects.Count < 1)
                return null;
            if (applicationObject.SelectedItems.Count == 1 && applicationObject.SelectedItems.Item(1).Project != null)
                return applicationObject.SelectedItems.Item(1).Project;
            if (applicationObject.SelectedItems.Count == 1 && applicationObject.SelectedItems.Item(1).ProjectItem.ContainingProject != null)
                return applicationObject.SelectedItems.Item(1).ProjectItem.ContainingProject;
            return null;
        }
        //http://msdn.microsoft.com/en-us/library/envdte._dte.activesolutionprojects.aspx
        public static Project GetActiveSolutionProject(this DTE2 applicationObject)
        {
            try
            {
                if (applicationObject.Solution == null || applicationObject.Solution.Projects == null || applicationObject.Solution.Projects.Count < 1)
                    return null;
                Array projectItems = (Array)applicationObject.ActiveSolutionProjects;
                if (projectItems.Length == 1 && projectItems.GetValue(0) != null)
                {
                    return ((Project)projectItems.GetValue(0));
                }
            }
            catch
            {
            }
            return null;
        }
        //see http://stackoverflow.com/questions/1434720/get-a-list-of-solution-project-files-for-vs-add-in-or-dxcore-plugin
        public static Project GetProjectbyInfo(this DTE2 applicationObject, IProjectInfo info)
        {
            try
            {
                if (info == null || applicationObject == null || applicationObject.Solution == null
                    || applicationObject.Solution.Projects == null || applicationObject.Solution.Projects.Count < 1)
                    return null;
                var projects = applicationObject.Solution.Projects.OfType<Project>();
                return projects.SingleOrDefault(prj => 
                    {
                        try
                        {
                            return string.Compare(prj.FileName, info.FileName, true) == 0;
                        }
                        catch (Exception)
                        {//catch Not implemented exceptions and continue, shows in VS 2013

                            return false;
                        }
                    });

            }
            catch(Exception ex)
            {
                Trace.TraceInformation("Exception in GetProjectbyInfo: {0}", ex.Message);
            }
            return null;
        }
        public static bool IsSolutionLoaded(this DTE2 applicationObject)
        {
            if (applicationObject == null)
                return false;
            return applicationObject.Solution.IsOpen;

        }
        //see http://www.mztools.com/articles/2006/mz2006004.aspx
        public static ProjectItem FindProjectItemsByName(this DTE2 applicationObject, string fileName)
        {
            try
            {
                if (!applicationObject.Solution.IsOpen)
                {
                    MessageBox.Show("Please load or create a solution");
                }
                else
                {
                    return applicationObject.Solution.Projects.OfType<Project>().Select(p => p.FindProjectItemByName(fileName)).First(pi => pi != null);
                }
            }
            catch (System.Exception objException)
            {
                MessageBox.Show(objException.ToString());
            }
            return null;

        }
        public static void InsertText(this ProjectItem pi, string content)
        {
            if (pi == null || pi.Document == null)
                return;
            Document document = pi.Document;

            TextDocument editDoc = (TextDocument)document.Object("TextDocument");
            EditPoint objEditPt = editDoc.CreateEditPoint(null);
            objEditPt.StartOfDocument();
            document.ReadOnly = false;

            while (!objEditPt.AtEndOfDocument)
            {
                objEditPt.Delete(objEditPt.LineLength);
                objEditPt.LineDown(1);
            }

            objEditPt.DeleteWhitespace(vsWhitespaceOptions.vsWhitespaceOptionsHorizontal);
            objEditPt.DeleteWhitespace(vsWhitespaceOptions.vsWhitespaceOptionsVertical);
            objEditPt.Insert(content);
            Trace.TraceInformation("saving file {0}", document.FullName);
            document.Save(document.FullName);
        }
        public static bool HasFile(this Project proj, string fileName)
        {
            if (proj == null || string.IsNullOrEmpty(fileName))
                return false;
            return DTEUtils.FindFile(Path.GetDirectoryName(proj.FullName), fileName);
        }
        public static bool DeleteProjectItem(this Project proj, string itemName)
        {
            if (proj == null || string.IsNullOrEmpty(itemName))
                return false;
            try
            {
                ProjectItem pi = proj.ProjectItems.FindProjectItemByName(itemName);

                if (pi.Document != null)
                {
                    if (pi.Document.ActiveWindow != null)
                        pi.Document.ActiveWindow.Close(vsSaveChanges.vsSaveChangesNo);
                }
                //the doc can be set to null by events fired
                if (pi.Document != null)
                    pi.Document.Close(vsSaveChanges.vsSaveChangesNo);

                pi.Remove();
                string fn = pi.get_FileNames(0);
                if (File.Exists(fn))
                    File.Delete(fn);
                return true;
            }
            catch (Exception)
            {

                return false;
            }
        }

        public static IProjectInfo GetProjectInfoDTE(this Project proj)
        {

            try
            {
                DesignPatterns.ProjectInfoDTE prjInf = new DesignPatterns.ProjectInfoDTE(proj);
                return prjInf;
            }
            catch
            {
                return null;
            }

        }

        public static IEnumerable<Property> GetNestedProperties(this Properties props)
        {
            return props.OfType<Property>().Where(p =>
            {
                try
                {
                    Properties prop = p.Value as Properties;
                    return prop != null;
                }
                catch (Exception)
                {
                    return false;
                }
            });
        }
        public static IEnumerable<Property> GetFlatProperties(this Properties props)
        {
            return props.OfType<Property>().Where(p =>
            {
                try
                {
                    Property prop = p.Value as Property;
                    return prop != null;
                }
                catch (Exception)
                {
                    return false;
                }
            });
        }

        public static Dictionary<string, string> GetProjectProperties(this Project proj)
        {
#if DEBUG
            var properties = proj.Properties.OfType<Property>().OrderBy(q => q.Name);
#else
            var properties = proj.Properties.OfType<Property>();
#endif
            return properties.ToDictionary(p => p.Name, p =>
            {
                try
                {
                    return p.Value as string;
                }
                catch
                {
                    return null;
                }
            });
        }


        public static IWizardWindow CreateWizardWindowFromProject(this Project proj, IWin32Window vsWindow)
        {
            try
            {
                if (proj == null)//|| proj.GuessProjectFileExtension() == null)
                    return null;

                if (proj.GuessProjectType() == CodeLangEnum.Unsupported)
                {
                    MessageBox.Show(vsWindow, string.Format("the project's {0} '{1}' language is not supported", proj.Name, proj.CodeModel.Language));
                    return null;
                }

                IProjectInfo prjInf = proj.GetProjectInfoDTE().GetAsService<IProjectInfo>();
                WizardForm frm = new WizardForm(prjInf);
                frm.VSWindow = vsWindow;
                return frm;
            }
            catch
            {

                return null;
            }
        }

        public static ProjectItem FindProjectItemByName(this Project objProject, string fileName)
        {

            string sMsg = null;
            ProjectItem projectItem = default(ProjectItem);

            sMsg = "Project Name: " + objProject.Name;
            sMsg += Environment.NewLine + "Kind: " + GuessProjectType(objProject);//DecodeProjectKind(objProject.Kind);

            try
            {
                projectItem = objProject.ParentProjectItem;
            }
            catch
            {
            }

            if ((projectItem != null))
            {
                sMsg += Environment.NewLine + "Parent Project Item: " + projectItem.Name;
            }
            else
            {
                sMsg += Environment.NewLine + "Parent Project Item: None";
            }

            Trace.TraceInformation(sMsg);

            return FindProjectItemByName(objProject.ProjectItems, fileName);

        }


        public static ProjectItem FindProjectItemByName(this ProjectItems colProjectItems, string fileName)
        {
            string sMsg = null;

            if ((colProjectItems != null))
            {

                foreach (ProjectItem objProjectItem in colProjectItems)
                {

                    sMsg = "Project Item Name: " + objProjectItem.Name;
                    sMsg += Environment.NewLine + "Kind: " + DecodeProjectItemKind(objProjectItem.Kind);
                    sMsg += Environment.NewLine + "Containing Project: " + objProjectItem.ContainingProject.Name;
                    if (objProjectItem.Collection.Parent is Project)
                    {
                        sMsg += Environment.NewLine + "Parent (project): " + ((Project)objProjectItem.Collection.Parent).Name;
                    }
                    else if (objProjectItem.Collection.Parent is ProjectItem)
                    {
                        sMsg += Environment.NewLine + "Parent (project item): " + ((ProjectItem)objProjectItem.Collection.Parent).Name;
                    }
                    Trace.TraceInformation(sMsg);
                    if (fileName == objProjectItem.Name)
                        return objProjectItem;
                    if ((objProjectItem.SubProject != null))
                    {
                        // We navigate recursively because it can be:
                        // - An Enterprise project in Visual Studio .NET 2002/2003
                        // - A solution folder in VS 2005
                        ProjectItem pi = FindProjectItemByName(objProjectItem.SubProject, fileName);
                        if (pi != null)
                            return pi;
                    }
                    else
                    {
                        // We navigate recursively because it can be:
                        // - An folder inside a project
                        // - A project item with nested project items (code-behind files, etc.)
                        ProjectItem pi = FindProjectItemByName(objProjectItem.ProjectItems, fileName);
                        if (pi != null)
                            return pi;

                    }

                }

            }

            return null;
        }

        public static CodeLangEnum GuessProjectType(this Project project)
        {
            try
            {
                //F# {f2a71f9b-5d33-465a-a702-920d77279786}
                if (project != null && Path.GetExtension(project.FileName).ToLower() == ".fsproj")
                    return CodeLangEnum.FSharp;
                if (project == null || project.CodeModel == null || project.CodeModel.Language == null)
                    return CodeLangEnum.Unsupported;
                switch (project.CodeModel.Language)
                {

                    case EnvDTE.CodeModelLanguageConstants.vsCMLanguageVB:
                        return CodeLangEnum.VB;

                    case EnvDTE.CodeModelLanguageConstants.vsCMLanguageCSharp:
                        return CodeLangEnum.CSharp;//TypeScript projects are seen as C# projects too

                    //http://msdn.microsoft.com/en-us/library/ms228770(v=vs.80).aspx
                    case EnvDTE.CodeModelLanguageConstants.vsCMLanguageVC:
                        return CodeLangEnum.NativeCpp;


                    case EnvDTE.CodeModelLanguageConstants.vsCMLanguageMC:
                        return CodeLangEnum.ManagedCpp;

                    case EnvDTE80.CodeModelLanguageConstants2.vsCMLanguageJSharp:
                        // Only Visual Studio 2005
                        return CodeLangEnum.JSharp;

                    default:
                        //add something for F# and other projects
                        return CodeLangEnum.Unsupported;

                }

            }
            catch (System.Exception objException)
            {
                Trace.TraceError("Can't guess project/file type, exception{0}", objException);
                return CodeLangEnum.Unsupported;
            }

        }
        internal static bool FindFile(string startFolder, string filename)
        {

            // Take a snapshot of the file system.
            System.IO.DirectoryInfo dir = new System.IO.DirectoryInfo(startFolder);

            // This method assumes that the application has discovery permissions 
            // for all folders under the specified path.
            IEnumerable<System.IO.FileInfo> fileList = dir.GetFiles("*.*", System.IO.SearchOption.AllDirectories);
            return fileList.Any(fi => string.Compare(fi.Name, filename, true) == 0);

        }
        public static string DecodeProjectItemKind(string sProjectItemKind)
        {

            string sResult = null;

            switch (sProjectItemKind)
            {
                case EnvDTE.Constants.vsProjectItemKindMisc:
                    sResult = "EnvDTE.Constants.vsProjectItemKindMisc";
                    break;
                case EnvDTE.Constants.vsProjectItemKindPhysicalFile:
                    sResult = "EnvDTE.Constants.vsProjectItemKindPhysicalFile";
                    break;
                case EnvDTE.Constants.vsProjectItemKindPhysicalFolder:
                    sResult = "EnvDTE.Constants.vsProjectItemKindPhysicalFolder";
                    break;
                case EnvDTE.Constants.vsProjectItemKindSolutionItems:
                    sResult = "EnvDTE.Constants.vsProjectItemKindSolutionItems";
                    break;
                case EnvDTE.Constants.vsProjectItemKindSubProject:
                    sResult = "EnvDTE.Constants.vsProjectItemKindSubProject";
                    break;
                case EnvDTE.Constants.vsProjectItemKindVirtualFolder:
                    sResult = "EnvDTE.Constants.vsProjectItemKindVirtualFolder";
                    break;
                default:
                    sResult = "";
                    break;
            }

            return sResult;

        }
    }
}
