﻿//  Copyright © Decebal Mihailescu 2013
//  All rights reserved. This source code and binaries are distributed under 
//  Microsoft Public License (Ms-PL)
//  The latest code is available at http://VSDesignPatterns.codeplex.com

//  Redistributions of source code must retain the above copyright notice, 
//  this list of conditions and the following disclaimer. 
//  Redistributions in binary form must reproduce the above copyright notice, 
//  this list of conditions and the following disclaimer in the documentation 
//  and/or other materials provided with the distribution. 

//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
//  PURPOSE. IT CAN BE DISTRIBUTED FREE OF CHARGE AS LONG AS THIS HEADER 
//  REMAINS UNCHANGED.
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace DesignPatternsWizard
{
    public partial class ConfigForm : Form
    {
        public ConfigForm()
        {
            InitializeComponent();
        }
        float _assemblyVer;
        private void ConfigForm_Load(object sender, EventArgs e)
        {
            this.CenterToParent();
            LocationsEnum persistedLocation = BaseLanguageProvider.Locations;
            LoadEnumInCheckedListBox(persistedLocation);
            ColumnsEnum persistedColumns = BaseLanguageProvider.Columns;
            LoadEnumInCheckedListBox(persistedColumns);
            Version v = System.Reflection.Assembly.GetExecutingAssembly().GetName().Version;
            _assemblyVer = float.Parse(string.Format("{0}.{1}", v.Major, v.Minor), System.Globalization.NumberStyles.Float);
            this.Text += string.Format(" Assembly Version:{0}", _assemblyVer);
            IWizardWindow iww = this.Owner as IWizardWindow;
            IProjectInfo ipi = iww.GetAsService<IProjectInfo>();
            ProjectInfo pi = ipi as ProjectInfo;
            var providers = pi.LangProviders.Select(p => new { p.Metadata.LanguageProviderName, p.Metadata.Language, p.Metadata.LicenseType, p.Metadata.Description, Used = true }
            );
            _dataGridViewProviders.DataSource = providers.ToArray();
        }
        private void LoadEnumInCheckedListBox(LocationsEnum persistedLocation)
        {
            _checkedListBoxPositions.Items.Clear();
            Array ar = Enum.GetValues(typeof(LocationsEnum));
            IEnumerable<LocationsEnum> allLocations = ar.AsQueryable().Cast<LocationsEnum>().Skip(1);//skip 'None'
            foreach (LocationsEnum loc in allLocations)
            {
                _checkedListBoxPositions.Items.Add(loc, (persistedLocation & loc) != LocationsEnum.None);
            }
        }
        private void LoadEnumInCheckedListBox(ColumnsEnum persistedLocation)
        {
            _checkedListBoxColumns.Items.Clear();
            ColumnsEnum[] allLocations = ProjectUtils.GetEnumValues<ColumnsEnum>();//ar.AsQueryable().Cast<ColumnsEnum>();
            foreach (ColumnsEnum loc in allLocations)
            {
                _checkedListBoxColumns.Items.Add(loc, (persistedLocation & loc) != default(ColumnsEnum));
            }
        }

        bool _bCancelPos;
        private void _buttonOK_Click(object sender, EventArgs e)
        {
            _bCancelPos = false;
            try
            {
                LocationsEnum ckl = _checkedListBoxPositions.CheckedItems.Cast<LocationsEnum>().Aggregate((acc, loc) => loc | acc);
                BaseLanguageProvider.Locations = ckl;
                if (_locationsChanged)
                    MessageBox.Show(this, "Changes to the Positions will be available\r\nonly after you restart Visual Studio.", "Fair Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);

            }
            catch (InvalidOperationException iex)
            {
                MessageBox.Show(this,
                    "You must leave at least one position available\r\nin order to continue using this add in.",
                    iex.Message, MessageBoxButtons.OK, MessageBoxIcon.Stop);
                _bCancelPos = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Exception thrown", MessageBoxButtons.OK, MessageBoxIcon.Error);
                BaseLanguageProvider.Locations = LocationsEnum.ContextProject;
                _bCancelPos = true;
            }


            try
            {
                var cols = _checkedListBoxColumns.CheckedItems.Cast<ColumnsEnum>();
                if (cols.Count() == 0)
                    BaseLanguageProvider.Columns = default(ColumnsEnum);
                else
                    BaseLanguageProvider.Columns = cols.Aggregate((acc, loc) => loc | acc);
            }
            catch
            {
                BaseLanguageProvider.Columns = default(ColumnsEnum);
            }


        }
        bool _locationsChanged;
        private void _checkedListBoxPositions_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            ListBox lb = sender as ListBox;
            if (lb != null && lb.SelectedItem != null && lb.SelectedItem.ToString().Length > 0)
            {
                _buttonOK.Enabled = true;
                if (sender == _checkedListBoxPositions)
                    _locationsChanged = true;
            }
        }

        private void ConfigForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = _bCancelPos;
        }

        private void _buttonRestoreDefault_Click(object sender, EventArgs e)
        {
            switch (_tabControl.SelectedIndex)
            {
                case 0:
                    ColumnsEnum persistedColumns = BaseLanguageProvider.DefaultColumns;
                    LoadEnumInCheckedListBox(persistedColumns);
                    break;
                case 1:
                    LocationsEnum persistedLocation = BaseLanguageProvider.DefaultLocations;
                    LoadEnumInCheckedListBox(persistedLocation);
                    break;
                default:
                    break;
            }
        }
        /*
        private void _buttonCheckVersion_Click(object sender, EventArgs e)
        {
            try
            {

                this.UseWaitCursor = true;
                Application.DoEvents();
                string downloadUrl;
                float onlineVersion;
                ProjectUtils.LastOnlineCheck = DateTime.Today;
                if (ProjectUtils.NewOnlineVersionExists(out downloadUrl, out onlineVersion))
                {
                    if (MessageBox.Show(this, "Version " + onlineVersion.ToString() + " is available online.\r\nWould you like to download it?", "A new version is available", MessageBoxButtons.YesNo) == DialogResult.Yes)
                    {
                        System.Diagnostics.Process.Start(string.IsNullOrEmpty(downloadUrl) ? ProjectUtils.BaseURL : downloadUrl);
                    }
                }
                else
                    if (onlineVersion > _assemblyVer)//not sure
                        System.Diagnostics.Process.Start(ProjectUtils.BaseURL);
                    else
                        MessageBox.Show(this, "There is no new version available online.\r\nCheck back in a few days.", "No new version is available");

            }
            catch //(Exception ex)
            {
                
            }
            finally
            {
                UseWaitCursor = false;
            }
        }
        */

    }
}
